import com.fasterxml.jackson.core.*;
import org.codehaus.jackson.map.JsonMappingException;
import org.joda.time.DateTime;

import java.io.File;
import java.io.IOException;

/**
 * Created by magnus on 11.11.15.
 *
 * This program takes a file as an argument and creates the fields
 * Campus, Building, Floor, HourOfDay and DayOfWeek. It also removes the Hierarchy field.
 */
public class AddNewFieldsToDataSet {
    private double maxLat= 10.41134;
    private double minLat= 10.39873;
    private double maxLong= 63.41992;
    private double minLong= 63.41372;

    public static void main(String [] args){
        AddNewFieldsToDataSet dc = new AddNewFieldsToDataSet();
        dc.init();
    }
    public void init(){
        addNewFieldsToDataSet("/path/to/file", "target_file_name", "new_file");
    }
    protected void addNewFieldsToDataSet(String path, String sourceFile, String outputFile){
        try {
            JsonFactory jsonfactory = new JsonFactory();
            File source = new File (path+sourceFile);
            System.out.println("Reading file: "+source.getName());
            JsonParser parser = jsonfactory.createParser(source);
            JsonFactory jfactory = new JsonFactory();
            /*** write to file ***/
            JsonGenerator jGenerator = jfactory.createGenerator(new File(
                    path+outputFile), JsonEncoding.UTF8);
            jGenerator.writeStartArray(); // [
            while (parser.nextToken()!=JsonToken.END_ARRAY) {
                jGenerator.writeStartObject(); // {
                while (parser.nextToken() != JsonToken.END_OBJECT){
                    String token = parser.getCurrentName();
                    if("hierarchy".equals(token)){
                        parser.nextToken();
                        String hierarchy = parser.getText();
                        String [] split = hierarchy.split(">");
                        jGenerator.writeStringField("campus", split[0]);
                        jGenerator.writeStringField("building", split[1]);
                        jGenerator.writeStringField("floor", split[2]);
                    }
                    else if ("timestamp".equals(token)) {
                        parser.nextToken();
                        long timestamp = parser.getLongValue();
                        int dayOfWeek = getDayOfWeek(timestamp);
                        int hourOfDay = getHourOfDay(timestamp);
                        jGenerator.writeNumberField("timestamp", timestamp);
                        jGenerator.writeNumberField("dayOfWeek", dayOfWeek);
                        jGenerator.writeNumberField("hourOfDay", hourOfDay);
                    }
                    else if ("longitude".equals(token)) {
                        parser.nextToken();
                        double latitude = parser.getDoubleValue();
                        jGenerator.writeNumberField("latitude", latitude);
                    }
                    else if ("latitude".equals(token)) {
                        parser.nextToken();
                        double longitude = parser.getDoubleValue();
                        jGenerator.writeNumberField("longitude", longitude);
                    }
                    else if ("salt_timestamp".equals(token)) {
                        parser.nextToken();
                        long salt = parser.getLongValue();
                        jGenerator.writeNumberField("salt_timestamp", salt);
                    }
                    else if ("id".equals(token)) {
                        parser.nextToken();
                        String id = parser.getText();
                        jGenerator.writeStringField("id", id);
                    }
                    else if ("accuracy".equals(token)) {
                        parser.nextToken();
                        double accuracy = parser.getDoubleValue();
                        jGenerator.writeNumberField("accuracy", accuracy);
                    }
                }
                jGenerator.writeEndObject(); // }
            }
            jGenerator.writeEndArray();
            jGenerator.close();
        }catch (JsonGenerationException e) {
            e.printStackTrace();
        }catch (JsonMappingException e) {
            e.printStackTrace();
        }catch (IOException e){
            e.printStackTrace();
        }
        System.out.println("FINISHED with "+ sourceFile+"!");
    }
    protected int getDayOfWeek(long timestamp){
        DateTime dateTime = new DateTime(timestamp*1000L);
        return dateTime.getDayOfWeek();
    }
    protected int getHourOfDay(long timestamp){
        DateTime dateTime = new DateTime(timestamp*1000L);
        return dateTime.getHourOfDay();
    }
}
