import org.json.simple.JSONObject;

import java.util.Arrays;

/**
 * Created by magnus on 29.10.15.
 *
 * This program takes a double two-dimensional matrix as an argument and compresses the matrix
 * to a CRS matrix and stores it as a JSON.
 */

public class CRS{
    private double maxLat= 63.41992;  //specify the maximum latitude coordinate
    private double minLat= 63.41372;  //specify the minimum latitude coordinate
    private double maxLong= 10.41134; //specify the maximum longitude coordinate
    private double minLong= 10.39873; //specify the minimum longitude coordinate

    protected JSONObject convertToCRS(double[][] matrix){
        double[] val;
        int [] col_idx;
        int [] row_ptr;
        int i;//row index
        int j;//column index
        int totalNonZeros;//the total number of nonzero in the matrix
        int index;
        int colSize = matrix.length;
        int rowSize = matrix[0].length;
        totalNonZeros = 0;
        for(i=0; i<rowSize; i++){//each row
            for(j=0; j<colSize; j++){//each column
                if(matrix[j][i] != 0){
                    totalNonZeros++;
                }
            }
        }
        val = new double[totalNonZeros];
        col_idx = new int[totalNonZeros];
        row_ptr = new int[colSize+1];
        row_ptr[0]= 0;
        index = 0;// point to the next position in val to store the value
        for(i=0; i<rowSize; i++){
            for(j=0; j<colSize; j++){
                if(matrix[j][i] != 0){
                    //add the value to val
                    val[index] = matrix[j][i];
                    //record the column index in col_idx
                    col_idx[index] = j;
                    index++;
                }
            }
            row_ptr[i+1] = index;
        }
        JSONObject obj = storeCSRMatrixAsJSON(colSize, rowSize,val, col_idx, row_ptr);
        return obj;
    }
    private JSONObject storeCSRMatrixAsJSON(int col_size, int row_size, double[]val, int[]col_idx,int[] row_ptr){
        JSONObject jo = new JSONObject();
        jo.put("col_size",col_size);
        jo.put("row_size",row_size);
        jo.put("maxLat",maxLat);
        jo.put("maxLong",maxLong);
        jo.put("minLat",minLat);
        jo.put("minLong",minLong);
        jo.put("val", Arrays.toString(val));
        jo.put("col",Arrays.toString(col_idx));
        jo.put("row",Arrays.toString(row_ptr));
        return jo;
    }
}