import com.mongodb.Block;
import com.mongodb.client.FindIterable;
import org.bson.Document;
import org.json.simple.JSONObject;

/**
 * Created by magnus on 19.05.16.
 *
 * This example uses MongoDB to iterate over the documents to create a two-dimensional matrix.
 * Change what the methods createMatrixV1 and createMatrixV2 is taking as arguments and how
 * the argument is iterated over to support the desired format.
 */
public class Matrix {

    private CRS crs = new CRS();
    private double maxLat= 63.41992;  //specify the maximum latitude coordinate
    private double minLat= 63.41372;  //specify the minimum latitude coordinate
    private double maxLong= 10.41134; //specify the maximum longitude coordinate
    private double minLong= 10.39873; //specify the minimum longitude coordinate

    protected JSONObject createMatrixV2(FindIterable<Document> iterable) {
        int xx = 250;
        int yy = 150;
        final double[][] matrix = new double[xx][yy];
        final int minLongValue = (int)(minLong * 100000);
        final int maxLongValue = (int)(maxLong * 100000);
        final int minLatValue = (int)(minLat * 100000);
        final int maxLatValue = (int)(maxLat * 100000);

        final double longRange = maxLongValue - minLongValue;
        final double latRange = maxLatValue - minLatValue;
        final double longByMatrixRange = longRange/xx;
        final double latByMatrixRange = latRange/yy;

        iterable.forEach(new Block<Document>() {
            int tel = 0;
            int totalTime=0;
            Long s = System.nanoTime();

            @Override
            public void apply(final Document document) {
                int longValue = (int) ((Double.parseDouble(document.get("longitude").toString())) * 100000);
                int latValue = (int) ((Double.parseDouble(document.get("latitude").toString())) * 100000);

                int longDifference = longValue-minLongValue;
                int latDifference = maxLatValue - latValue;
                int x = (int)(longDifference/longByMatrixRange);
                int y = (int)(latDifference/latByMatrixRange);

                if (x >= matrix.length || x < 0 || y >= matrix[0].length || y < 0) {
                } else {
                    try {
                        matrix[x][y] += 1;
                    }catch (Exception e){
                        e.printStackTrace();
                        System.out.println(x + " - " + y);
                    }
                }
            }
        });
        JSONObject obj = crs.convertToCRS(matrix);
        return obj;
    }
    protected JSONObject createMatrixV1(FindIterable<Document> iterable) {
        int xx = (int) ((maxLong * 100000) - (minLong * 100000));
        int yy = (int) ((maxLat * 100000) - (minLat * 100000));
        final double[][] matrix = new double[xx][yy];

        iterable.forEach(new Block<Document>() {
            @Override
            public void apply(final Document document) {
                int tempLong = (int) ((Double.parseDouble(document.get("longitude").toString())) * 100000);
                int x = (int) (maxLong * 100000) - tempLong;
                int tempLat = (int) ((Double.parseDouble(document.get("latitude").toString())) * 100000);
                int y = (int) (maxLat * 100000) - tempLat;
                //extra test to avoid out of bounds
                if (x > matrix.length || x < 0 || y > matrix[0].length || y < 0) {
                } else {
                    matrix[x][y] += 1;
                }
            }
        });
        JSONObject obj = crs.convertToCRS(matrix);
        return obj;
    }
}
